@@ -2,6 +2,7 @@ use std::path::PathBuf;
 
 #[derive(Debug)]
 pub enum FxError {
+    TerminalSizeDetection,
     Io(String),
     Dirs(String),
     GetItem,
@@ -15,8 +16,10 @@ pub enum FxError {
     RemoveItem(PathBuf),
     TooSmallWindowSize,
     Log(String),
-    Panic,
     Unpack(String),
+    Panic,
+    #[cfg(any(target_os = "linux", target_os = "netbsd"))]
+    Nix(String),
 }
 
 impl std::error::Error for FxError {}
@@ -24,6 +27,7 @@ impl std::error::Error for FxError {}
 impl std::fmt::Display for FxError {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
         let printable = match self {
+            FxError::TerminalSizeDetection => "Error: Cannot detect terminal size.".to_owned(),
             FxError::Io(s) => s.to_owned(),
             FxError::Dirs(s) => s.to_owned(),
             FxError::GetItem => "Error: Cannot get item info".to_owned(),
@@ -37,8 +41,10 @@ impl std::fmt::Display for FxError {
             FxError::RemoveItem(s) => format!("Error: Cannot remove -> {:?}", s),
             FxError::TooSmallWindowSize => "Error: Too small window size".to_owned(),
             FxError::Log(s) => s.to_owned(),
-            FxError::Panic => "Error: felix panicked".to_owned(),
             FxError::Unpack(s) => s.to_owned(),
+            FxError::Panic => "Error: felix panicked".to_owned(),
+            #[cfg(any(target_os = "linux", target_os = "netbsd"))]
+            FxError::Nix(s) => s.to_owned(),
         };
         write!(f, "{}", printable)
     }
@@ -84,3 +90,10 @@ impl From<zip::result::ZipError> for FxError {
         FxError::Unpack(err.to_string())
     }
 }
+
+#[cfg(any(target_os = "linux", target_os = "netbsd"))]
+impl From<nix::errno::Errno> for FxError {
+    fn from(err: nix::errno::Errno) -> Self {
+        FxError::Nix(err.to_string())
+    }
+}
