@@ -1,35 +1,37 @@
 use super::config::*;
 use super::errors::FxError;
 use super::functions::*;
+use super::help::HELP;
 use super::layout::*;
 use super::magic_image::is_supported_image_type;
+use super::magic_packed;
 use super::nums::*;
 use super::op::*;
 use super::session::*;
 use super::term::*;
 
 use chrono::prelude::*;
-use crossterm::event;
-use crossterm::event::{KeyCode, KeyEvent};
+use crossterm::event::{Event, KeyCode, KeyEvent};
 use crossterm::style::Stylize;
 use log::{error, info};
-use std::collections::HashMap;
-use std::collections::HashSet;
+use std::collections::{BTreeMap, BTreeSet};
 use std::env;
 use std::ffi::OsStr;
-use std::fmt::Write as _;
 use std::fs;
-#[cfg(target_family = "unix")]
-use std::os::unix::fs::PermissionsExt;
+use std::io::Stdout;
+use std::io::Write;
 use std::path::PathBuf;
-use std::process::{Child, Command, ExitStatus, Stdio};
+use std::process::{Command, ExitStatus, Stdio};
 use std::time::UNIX_EPOCH;
 use syntect::highlighting::{Theme, ThemeSet};
 
+#[cfg(target_family = "unix")]
+use std::os::unix::fs::PermissionsExt;
+
+pub const FELIX: &str = "felix";
 pub const BEGINNING_ROW: u16 = 3;
-pub const FX_CONFIG_DIR: &str = "felix";
-pub const TRASH: &str = "trash";
 pub const EMPTY_WARNING: &str = "Are you sure to empty the trash directory? (if yes: y)";
+const TIME_PREFIX: usize = 11;
 
 #[derive(Debug)]
 pub struct State {
@@ -37,17 +39,16 @@ pub struct State {
     pub current_dir: PathBuf,
     pub trash_dir: PathBuf,
     pub default: String,
-    pub commands: Option<HashMap<String, String>>,
+    pub commands: Option<BTreeMap<String, String>>,
     pub registered: Vec<ItemInfo>,
     pub operations: Operation,
     pub c_memo: Vec<StateMemo>,
     pub p_memo: Vec<StateMemo>,
     pub keyword: Option<String>,
     pub layout: Layout,
-    pub rust_log: Option<String>,
 }
 
-#[derive(Debug, PartialEq, Eq, PartialOrd, Ord, Clone)]
+#[derive(Default, Debug, PartialEq, Eq, PartialOrd, Ord, Clone)]
 pub struct ItemInfo {
     pub file_type: FileType,
     pub file_name: String,
@@ -72,17 +73,23 @@ pub enum FileType {
     Symlink,
 }
 
+impl Default for FileType {
+    fn default() -> Self {
+        FileType::File
+    }
+}
+
 impl State {
     /// Initialize the state of the app.
-    pub fn new(p: &std::path::Path) -> Result<Self, FxError> {
+    pub fn new(p: &std::path::Path, session_path: &std::path::Path) -> Result<Self, FxError> {
         let config = match read_config(p) {
             Ok(c) => c,
             Err(e) => {
                 eprintln!("Cannot read the config file properly.\nError: {}\nDo you want to use the default config? [press Enter to continue]", e);
                 enter_raw_mode();
                 loop {
-                    match event::read()? {
-                        event::Event::Key(KeyEvent { code, .. }) => match code {
+                    match crossterm::event::read()? {
+                        Event::Key(KeyEvent { code, .. }) => match code {
                             KeyCode::Enter => break,
                             _ => {
                                 leave_raw_mode();
@@ -98,9 +105,8 @@ impl State {
                 Config::default()
             }
         };
-        let session = read_session()?;
-        let (original_column, original_row) =
-            crossterm::terminal::size().unwrap_or_else(|_| panic!("Cannot detect terminal size."));
+        let session = read_session(session_path)?;
+        let (original_column, original_row) = terminal_size()?;
 
         // Return error if terminal size may cause panic
         if original_column < 4 {
@@ -166,7 +172,6 @@ impl State {
             c_memo: Vec::new(),
             p_memo: Vec::new(),
             keyword: None,
-            rust_log: std::env::var("RUST_LOG").ok(),
         })
     }
 
@@ -209,8 +214,56 @@ impl State {
         }
     }
 
+    #[cfg(any(target_os = "linux", target_os = "netbsd"))]
     /// Open the selected file in a new window, according to the config.
-    pub fn open_file_in_new_window(&self) -> Result<Child, FxError> {
+    pub fn open_file_in_new_window(&self) -> Result<(), FxError> {
+        let item = self.get_item()?;
+        let path = &item.file_path;
+        let map = &self.commands;
+        let extension = &item.file_ext;
+
+        info!("OPEN(new window): {:?}", path);
+
+        match map {
+            None => Err(FxError::OpenNewWindow("No exec configuration".to_owned())),
+            Some(map) => match extension {
+                Some(extension) => match map.get(extension) {
+                    Some(command) => match unsafe { nix::unistd::fork() } {
+                        Ok(result) => match result {
+                            nix::unistd::ForkResult::Parent { child } => {
+                                nix::sys::wait::waitpid(Some(child), None)?;
+                                Ok(())
+                            }
+                            nix::unistd::ForkResult::Child => {
+                                nix::unistd::setsid()?;
+                                let mut ex = Command::new(command);
+                                ex.arg(path)
+                                    .stdout(Stdio::null())
+                                    .stdin(Stdio::null())
+                                    .spawn()
+                                    .and(Ok(()))
+                                    .map_err(|_| FxError::OpenItem)?;
+                                drop(ex);
+                                std::process::exit(0);
+                            }
+                        },
+                        Err(e) => Err(FxError::Nix(e.to_string())),
+                    },
+                    None => Err(FxError::OpenNewWindow(
+                        "Cannot open this type of item in new window".to_owned(),
+                    )),
+                },
+
+                None => Err(FxError::OpenNewWindow(
+                    "Cannot open this type of item in new window".to_owned(),
+                )),
+            },
+        }
+    }
+
+    #[cfg(all(not(target_os = "linux"), not(target_os = "netbsd")))]
+    /// Open the selected file in a new window, according to the config.
+    pub fn open_file_in_new_window(&self) -> Result<(), FxError> {
         let item = self.get_item()?;
         let path = &item.file_path;
         let map = &self.commands;
@@ -228,6 +281,7 @@ impl State {
                             .stdout(Stdio::null())
                             .stdin(Stdio::null())
                             .spawn()
+                            .and(Ok(()))
                             .or(Err(FxError::OpenItem))
                     }
                     None => Err(FxError::OpenNewWindow(
@@ -252,8 +306,8 @@ impl State {
         for (i, item) in targets.iter().enumerate() {
             let item = item.clone();
 
-            print!(" ");
-            to_info_bar();
+            delete_cursor();
+            to_info_line();
             clear_current_line();
             print!("{}", display_count(i, total_selected));
 
@@ -269,12 +323,16 @@ impl State {
                         Err(e) => {
                             return Err(e);
                         }
-                        Ok(path) => trash_vec.push(path),
+                        Ok(path) => {
+                            if let Some(p) = path {
+                                trash_vec.push(p);
+                            }
+                        }
                     }
                 }
             }
         }
-        if new_op {
+        if new_op && !trash_vec.is_empty() {
             self.operations.branch();
             //push deleted item information to operations
             self.operations.push(OpKind::Delete(DeletedFiles {
@@ -288,14 +346,18 @@ impl State {
     }
 
     /// Move single file to trash directory.
-    fn remove_and_yank_file(&mut self, item: ItemInfo, new_op: bool) -> Result<PathBuf, FxError> {
+    fn remove_and_yank_file(
+        &mut self,
+        item: ItemInfo,
+        new_op: bool,
+    ) -> Result<Option<PathBuf>, FxError> {
         //prepare from and to for copy
         let from = &item.file_path;
         let mut to = PathBuf::new();
 
         if item.file_type == FileType::Symlink && !from.exists() {
-            match Command::new("rm").arg(from).status() {
-                Ok(_) => Ok(PathBuf::new()),
+            match std::fs::remove_file(from) {
+                Ok(_) => Ok(None),
                 Err(_) => Err(FxError::RemoveItem(from.to_owned())),
             }
         } else {
@@ -320,7 +382,7 @@ impl State {
                 return Err(FxError::RemoveItem(from.to_owned()));
             }
 
-            Ok(to)
+            Ok(Some(to))
         }
     }
 
@@ -366,6 +428,12 @@ impl State {
 
                     continue;
                 } else {
+                    if entry.file_type().is_symlink() && !entry_path.exists() {
+                        if std::fs::remove_file(entry_path).is_err() {
+                            return Err(FxError::RemoveItem(entry_path.to_owned()));
+                        }
+                        continue;
+                    }
                     target = entry_path.iter().skip(base).collect();
                     target = trash_path.join(target);
                     if entry.file_type().is_dir() {
@@ -412,8 +480,8 @@ impl State {
             for item in self.list.iter_mut().filter(|item| item.selected) {
                 self.registered.push(item.clone());
             }
-        } else {
-            let item = self.get_item().unwrap().clone();
+        } else if let Ok(item) = self.get_item() {
+            let item = item.clone();
             self.registered.push(item);
         }
     }
@@ -426,7 +494,7 @@ impl State {
         target_dir: Option<PathBuf>,
     ) -> Result<(), FxError> {
         //make HashSet<String> of file_name
-        let mut name_set = HashSet::new();
+        let mut name_set = BTreeSet::new();
         match &target_dir {
             None => {
                 for item in self.list.iter() {
@@ -451,8 +519,8 @@ impl State {
 
         let total_selected = targets.len();
         for (i, item) in targets.iter().enumerate() {
-            print!(" ");
-            to_info_bar();
+            delete_cursor();
+            to_info_line();
             clear_current_line();
             print!("{}", display_count(i, total_selected));
 
@@ -487,12 +555,12 @@ impl State {
         &mut self,
         item: &ItemInfo,
         target_dir: &Option<PathBuf>,
-        name_set: &mut HashSet<String>,
+        name_set: &mut BTreeSet<String>,
     ) -> Result<PathBuf, FxError> {
         match target_dir {
             None => {
                 if item.file_path.parent() == Some(&self.trash_dir) {
-                    let rename: String = item.file_name.chars().skip(11).collect();
+                    let rename: String = item.file_name.chars().skip(TIME_PREFIX).collect();
                     let rename = rename_file(&rename, name_set);
                     let to = &self.current_dir.join(&rename);
                     if std::fs::copy(&item.file_path, to).is_err() {
@@ -512,7 +580,7 @@ impl State {
             }
             Some(path) => {
                 if item.file_path.parent() == Some(&self.trash_dir) {
-                    let rename: String = item.file_name.chars().skip(11).collect();
+                    let rename: String = item.file_name.chars().skip(TIME_PREFIX).collect();
                     let rename = rename_file(&rename, name_set);
                     let to = path.join(&rename);
                     if std::fs::copy(&item.file_path, to.clone()).is_err() {
@@ -538,7 +606,7 @@ impl State {
         &mut self,
         buf: &ItemInfo,
         target_dir: &Option<PathBuf>,
-        name_set: &mut HashSet<String>,
+        name_set: &mut BTreeSet<String>,
     ) -> Result<PathBuf, FxError> {
         let mut base: usize = 0;
         let mut target: PathBuf = PathBuf::new();
@@ -563,11 +631,8 @@ impl State {
             if i == 0 {
                 base = entry_path.iter().count();
 
-                let parent = &original_path
-                    .parent()
-                    .ok_or_else(|| FxError::Io("Cannot read parent dir.".to_string()))?;
-                if parent == &self.trash_dir {
-                    let rename: String = buf.file_name.chars().skip(11).collect();
+                if original_path.parent() == Some(&self.trash_dir) {
+                    let rename: String = buf.file_name.chars().skip(TIME_PREFIX).collect();
                     target = match &target_dir {
                         None => self.current_dir.join(&rename),
                         Some(path) => path.join(&rename),
@@ -693,11 +758,10 @@ impl State {
     }
 
     /// Reload the app layout when terminal size changes.
-    pub fn refresh(&mut self, column: u16, row: u16, mut cursor_pos: u16) {
+    pub fn refresh(&mut self, column: u16, row: u16, mut cursor_pos: u16) -> Result<(), FxError> {
         let (time_start, name_max) = make_layout(column);
 
-        let (original_column, original_row) =
-            crossterm::terminal::size().unwrap_or_else(|_| panic!("Cannot detect terminal size."));
+        let (original_column, original_row) = terminal_size()?;
 
         self.layout.terminal_row = row;
         self.layout.terminal_column = column;
@@ -721,6 +785,7 @@ impl State {
         }
 
         self.redraw(cursor_pos);
+        Ok(())
     }
 
     /// Clear all and show the current directory information.
@@ -728,12 +793,24 @@ impl State {
         clear_all();
         move_to(1, 1);
 
+        let mut header_space = (self.layout.terminal_column - 1) as usize;
+
         //Show current directory path.
         //crossterm's Stylize cannot be applied to PathBuf,
         //current directory does not have any text attribute for now.
-        set_color(&TermColor::ForeGround(&Colorname::Cyan));
-        print!(" {}", self.current_dir.display(),);
-        reset_color();
+        let current_dir = self.current_dir.display().to_string();
+        if current_dir.bytes().len() >= header_space {
+            let current_dir = split_str(&current_dir, header_space);
+            set_color(&TermColor::ForeGround(&Colorname::Cyan));
+            print!(" {}", current_dir);
+            reset_color();
+            return;
+        } else {
+            set_color(&TermColor::ForeGround(&Colorname::Cyan));
+            print!(" {}", current_dir);
+            reset_color();
+            header_space -= current_dir.len();
+        }
 
         //If .git directory exists, get the branch information and print it.
         let git = self.current_dir.join(".git");
@@ -742,10 +819,13 @@ impl State {
             if let Ok(head) = std::fs::read(head) {
                 let branch: Vec<u8> = head.into_iter().skip(16).collect();
                 if let Ok(branch) = std::str::from_utf8(&branch) {
-                    print!(" on ",);
-                    set_color(&TermColor::ForeGround(&Colorname::LightMagenta));
-                    print!("{}", branch.trim().bold());
-                    reset_color();
+                    if branch.len() + 4 <= header_space {
+                        print!(" on ",);
+                        set_color(&TermColor::ForeGround(&Colorname::LightMagenta));
+                        print!("{}", branch.trim().bold());
+                        reset_color();
+                    } else {
+                    }
                 }
             }
         }
@@ -753,16 +833,13 @@ impl State {
 
     /// Print an item in the directory.
     fn print_item(&self, item: &ItemInfo) {
-        let chars: Vec<char> = item.file_name.chars().collect();
-        let name = if chars.len() > self.layout.name_max_len {
-            let mut result = chars
-                .iter()
-                .take(self.layout.name_max_len - 2)
-                .collect::<String>();
-            result.push_str("..");
-            result
-        } else {
+        let name = if item.file_name.bytes().len() <= self.layout.name_max_len {
             item.file_name.clone()
+        } else {
+            let i = (self.layout.name_max_len - 2) as usize;
+            let mut file_name = split_str(&item.file_name, i);
+            file_name.push_str("..");
+            file_name
         };
         let time = format_time(&item.modified);
         let color = match item.file_type {
@@ -843,8 +920,8 @@ impl State {
 
         match self.layout.sort_by {
             SortKey::Name => {
-                dir_v.sort_by(|a, b| natord::compare(&a.file_name, &b.file_name));
-                file_v.sort_by(|a, b| natord::compare(&a.file_name, &b.file_name));
+                dir_v.sort_by(|a, b| natord::compare_ignore_case(&a.file_name, &b.file_name));
+                file_v.sort_by(|a, b| natord::compare_ignore_case(&a.file_name, &b.file_name));
             }
             SortKey::Time => {
                 dir_v.sort_by(|a, b| b.modified.partial_cmp(&a.modified).unwrap());
@@ -863,6 +940,49 @@ impl State {
         Ok(())
     }
 
+    /// Change (only) the order of the list and print it.
+    pub fn reorder(&mut self, y: u16) {
+        self.change_order();
+        self.clear_and_show_headline();
+        self.list_up();
+        self.move_cursor(y);
+    }
+
+    /// Change the order of the list not reading all the items.
+    fn change_order(&mut self) {
+        let mut dir_v = Vec::new();
+        let mut file_v = Vec::new();
+        let mut result = Vec::with_capacity(self.list.len());
+
+        for item in self.list.iter_mut() {
+            if item.file_type == FileType::Directory {
+                dir_v.push(std::mem::take(item));
+            } else {
+                file_v.push(std::mem::take(item));
+            }
+        }
+
+        match self.layout.sort_by {
+            SortKey::Name => {
+                dir_v.sort_by(|a, b| natord::compare_ignore_case(&a.file_name, &b.file_name));
+                file_v.sort_by(|a, b| natord::compare_ignore_case(&a.file_name, &b.file_name));
+            }
+            SortKey::Time => {
+                dir_v.sort_by(|a, b| b.modified.partial_cmp(&a.modified).unwrap());
+                file_v.sort_by(|a, b| b.modified.partial_cmp(&a.modified).unwrap());
+            }
+        }
+
+        result.append(&mut dir_v);
+        result.append(&mut file_v);
+
+        if !self.layout.show_hidden {
+            result.retain(|x| !x.is_hidden);
+        }
+
+        self.list = result;
+    }
+
     /// Reset all item's selected state and exit the select mode.
     pub fn reset_selection(&mut self) {
         for mut item in self.list.iter_mut() {
@@ -902,6 +1022,83 @@ impl State {
         }
     }
 
+    //Show help
+    pub fn show_help(&self, mut screen: &Stdout) -> Result<(), FxError> {
+        clear_all();
+        move_to(1, 1);
+        screen.flush()?;
+        let (width, height) = terminal_size()?;
+        let help = format_txt(HELP, width, true);
+        print_help(&help, 0, height);
+        screen.flush()?;
+
+        let mut skip = 0;
+        loop {
+            if let Event::Key(KeyEvent { code, .. }) = crossterm::event::read()? {
+                match code {
+                    KeyCode::Char('j') | KeyCode::Down => {
+                        clear_all();
+                        skip += 1;
+                        print_help(&help, skip, height);
+                        screen.flush()?;
+                        continue;
+                    }
+                    KeyCode::Char('k') | KeyCode::Up => {
+                        if skip == 0 {
+                            continue;
+                        } else {
+                            clear_all();
+                            skip -= 1;
+                            print_help(&help, skip, height);
+                            screen.flush()?;
+                            continue;
+                        }
+                    }
+                    _ => {
+                        break;
+                    }
+                }
+            }
+        }
+        Ok(())
+    }
+
+    //Empty the trash dir
+    pub fn empty_trash(&mut self, mut screen: &Stdout) -> Result<(), FxError> {
+        print_warning(EMPTY_WARNING, self.layout.y);
+        screen.flush()?;
+
+        if let Event::Key(KeyEvent { code, .. }) = crossterm::event::read()? {
+            match code {
+                KeyCode::Char('y') | KeyCode::Char('Y') => {
+                    print_info("EMPTY: Processing...", self.layout.y);
+                    screen.flush()?;
+
+                    //Delete trash dir.
+                    if let Err(e) = std::fs::remove_dir_all(&self.trash_dir) {
+                        print_warning(e, self.layout.y);
+                    }
+                    //Recreate the dir.
+                    if let Err(e) = std::fs::create_dir(&self.trash_dir) {
+                        print_warning(e, self.layout.y);
+                    }
+                    if self.current_dir == self.trash_dir {
+                        self.reload(BEGINNING_ROW)?;
+                    }
+                    go_to_and_rest_info();
+                    print_info("Trash dir emptied", self.layout.y);
+                    self.move_cursor(self.layout.y);
+                    screen.flush()?;
+                }
+                _ => {
+                    go_to_and_rest_info();
+                    self.move_cursor(self.layout.y);
+                }
+            }
+        }
+        Ok(())
+    }
+
     pub fn chdir(&mut self, p: &std::path::Path, mv: Move) -> Result<(), FxError> {
         std::env::set_current_dir(p)?;
         match mv {
@@ -1084,7 +1281,7 @@ impl State {
     fn make_footer(&self, item: &ItemInfo) -> String {
         match &item.file_ext {
             Some(ext) => {
-                let mut footer = match item.permissions {
+                let footer = match item.permissions {
                     Some(permissions) => {
                         format!(
                             " {}/{} {} {} {}",
@@ -1103,23 +1300,13 @@ impl State {
                         to_proper_size(item.file_size),
                     ),
                 };
-                if self.rust_log.is_some() {
-                    let _ = write!(
-                        footer,
-                        " i:{} s:{} c:{} r:{}",
-                        self.layout.nums.index,
-                        self.layout.nums.skip,
-                        self.layout.terminal_column,
-                        self.layout.terminal_row
-                    );
-                }
                 footer
                     .chars()
                     .take(self.layout.terminal_column.into())
                     .collect()
             }
             None => {
-                let mut footer = match item.permissions {
+                let footer = match item.permissions {
                     Some(permissions) => {
                         format!(
                             " {}/{} {} {}",
@@ -1136,16 +1323,6 @@ impl State {
                         to_proper_size(item.file_size),
                     ),
                 };
-                if self.rust_log.is_some() {
-                    let _ = write!(
-                        footer,
-                        " i:{} s:{} c:{} r:{}",
-                        self.layout.nums.index,
-                        self.layout.nums.skip,
-                        self.layout.terminal_column,
-                        self.layout.terminal_row
-                    );
-                }
                 footer
                     .chars()
                     .take(self.layout.terminal_column.into())
@@ -1191,6 +1368,24 @@ impl State {
         fs::write(&session_path, serialized)?;
         Ok(())
     }
+
+    /// Unpack/unarchive a file.
+    pub fn unpack(&self) -> Result<(), FxError> {
+        let item = self.get_item()?;
+        let p = item.file_path.clone();
+
+        let mut name_set: BTreeSet<String> = BTreeSet::new();
+        for item in self.list.iter() {
+            name_set.insert(item.file_name.clone());
+        }
+
+        let dest_name = rename_dir(&item.file_name, &name_set);
+        let mut dest = self.current_dir.clone();
+        dest.push(dest_name);
+
+        magic_packed::unpack(&p, &dest)?;
+        Ok(())
+    }
 }
 
 /// Read item information from `std::fs::DirEntry`.
